@@ -301,14 +301,15 @@ _OPTS = [
                ' high number of revocation events combined with a low cache'
                ' duration may significantly reduce performance.'),
     cfg.StrOpt('memcache_security_strategy',
-               default=None,
+               default='None',
+               choices=('None', 'MAC', 'ENCRYPT'),
+               ignore_case=True,
                help='(Optional) If defined, indicate whether token data'
                ' should be authenticated or authenticated and encrypted.'
-               ' Acceptable values are MAC or ENCRYPT.  If MAC, token data is'
-               ' authenticated (with HMAC) in the cache. If ENCRYPT, token'
-               ' data is encrypted and authenticated in the cache. If the'
-               ' value is not one of these options or empty, auth_token will'
-               ' raise an exception on initialization.'),
+               ' If MAC, token data is authenticated (with HMAC) in the cache.'
+               ' If ENCRYPT, token data is encrypted and authenticated in the'
+               ' cache. If the value is not one of these options or empty,'
+               ' auth_token will raise an exception on initialization.'),
     cfg.StrOpt('memcache_secret_key',
                default=None,
                secret=True,
@@ -369,7 +370,7 @@ _OPTS = [
                 ' only while migrating from a less secure algorithm to a more'
                 ' secure one. Once all the old tokens are expired this option'
                 ' should be set to a single value for better performance.'),
-] + _auth.OPTS
+]
 
 CONF = cfg.CONF
 CONF.register_opts(_OPTS, group=_base.AUTHTOKEN_GROUP)
@@ -399,7 +400,7 @@ def _conf_values_type_convert(conf):
         return {}
 
     opt_types = {}
-    for o in _OPTS:
+    for o in _OPTS + _auth.OPTS:
         type_dest = (getattr(o, 'type', str), o.dest)
         opt_types[o.dest] = type_dest
         # Also add the deprecated name with the same type and dest.
@@ -428,7 +429,7 @@ def _get_project_version(project):
     return pkg_resources.get_distribution(project).version
 
 
-class _BaseAuthProtocol(object):
+class BaseAuthProtocol(object):
     """A base class for AuthProtocol token checking implementations.
 
     :param Callable app: The next application to call after middleware.
@@ -517,7 +518,7 @@ class _BaseAuthProtocol(object):
 
     def _do_fetch_token(self, token):
         """Helper method to fetch a token and convert it into an AccessInfo"""
-        data = self._fetch_token(token)
+        data = self.fetch_token(token)
 
         try:
             return data, access.create(body=data, auth_token=token)
@@ -525,7 +526,7 @@ class _BaseAuthProtocol(object):
             self.log.warning(_LW('Invalid token contents.'), exc_info=True)
             raise ksm_exceptions.InvalidToken(_('Token authorization failed'))
 
-    def _fetch_token(self, token):
+    def fetch_token(self, token):
         """Fetch the token data based on the value in the header.
 
         Retrieve the data associated with the token value that was in the
@@ -613,7 +614,7 @@ class _BaseAuthProtocol(object):
                 self._invalid_user_token()
 
 
-class AuthProtocol(_BaseAuthProtocol):
+class AuthProtocol(BaseAuthProtocol):
     """Middleware that handles authenticating client calls."""
 
     _SIGNING_CERT_FILE_NAME = 'signing_cert.pem'
@@ -647,8 +648,11 @@ class AuthProtocol(_BaseAuthProtocol):
                 default_config_files=default_config_files,
                 validate_default_values=True)
 
-            self._local_oslo_config.register_opts(
-                _OPTS, group=_base.AUTHTOKEN_GROUP)
+            self._local_oslo_config.register_opts(_OPTS,
+                                                  group=_base.AUTHTOKEN_GROUP)
+            self._local_oslo_config.register_opts(_auth.OPTS,
+                                                  group=_base.AUTHTOKEN_GROUP)
+
             loading.register_auth_conf_options(self._local_oslo_config,
                                                group=_base.AUTHTOKEN_GROUP)
 
@@ -798,49 +802,35 @@ class AuthProtocol(_BaseAuthProtocol):
         else:
             return [token]
 
-    def _cache_get_hashes(self, token_hashes):
-        """Check if the token is cached already.
-
-        Functions takes a list of hashes that might be in the cache and matches
-        the first one that is present. If nothing is found in the cache it
-        returns None.
-
-        :returns: token data if found else None.
-        """
-
-        for token in token_hashes:
-            cached = self._token_cache.get(token)
-
-            if cached:
-                return cached
-
-    def _fetch_token(self, token):
+    def fetch_token(self, token):
         """Retrieve a token from either a PKI bundle or the identity server.
 
         :param str token: token id
 
         :raises exc.InvalidToken: if token is rejected
         """
-        data = None
-        token_hashes = None
-
         try:
             token_hashes = self._token_hashes(token)
-            cached = self._cache_get_hashes(token_hashes)
+            offline_data = self._validate_offline(token, token_hashes)
+
+            if offline_data:
+                # NOTE(jamielennox): If we've validated a PKI token we don't
+                # need to cache it, and revocation check was already performed.
+                return offline_data
+
+            cached = self._token_cache.get_first(*token_hashes)
 
             if cached:
-                data = cached
-
                 if self._check_revocations_for_cached:
                     # A token might have been revoked, regardless of initial
                     # mechanism used to validate it, and needs to be checked.
                     self._revocations.check(token_hashes)
-            else:
-                data = self._validate_offline(token, token_hashes)
-                if not data:
-                    data = self._identity_server.verify_token(token)
 
-                self._token_cache.store(token_hashes[0], data)
+                return cached
+
+            data = self._identity_server.verify_token(token)
+            self._token_cache.store(token_hashes[0], data)
+            return data
 
         except (ksa_exceptions.ConnectFailure,
                 ksa_exceptions.RequestTimeout,
@@ -858,8 +848,6 @@ class AuthProtocol(_BaseAuthProtocol):
             self.log.critical(_LC('Unable to validate token'), exc_info=True)
             raise webob.exc.HTTPInternalServerError()
 
-        return data
-
     def _validate_offline(self, token, token_hashes):
         try:
             if cms.is_pkiz(token):
@@ -877,6 +865,18 @@ class AuthProtocol(_BaseAuthProtocol):
                                  'fallback to online validation.'))
         else:
             data = jsonutils.loads(verified)
+
+            audit_ids = None
+            if 'access' in data:
+                # It's a v2 token.
+                audit_ids = data['access']['token'].get('audit_ids')
+            else:
+                # It's a v3 token
+                audit_ids = data['token'].get('audit_ids')
+
+            if audit_ids:
+                self._revocations.check_by_audit_id(audit_ids)
+
             return data
 
     def _validate_token(self, auth_ref):
@@ -1077,12 +1077,18 @@ class AuthProtocol(_BaseAuthProtocol):
             requested_auth_version=auth_version)
 
     def _token_cache_factory(self):
+        memcached_servers = self._conf_get('memcached_servers')
+        env_cache_name = self._conf_get('cache')
+
+        if not (memcached_servers or env_cache_name):
+            return _cache.NoOpCache()
+
         security_strategy = self._conf_get('memcache_security_strategy')
 
         cache_kwargs = dict(
             cache_time=int(self._conf_get('token_cache_time')),
-            env_cache_name=self._conf_get('cache'),
-            memcached_servers=self._conf_get('memcached_servers'),
+            env_cache_name=env_cache_name,
+            memcached_servers=memcached_servers,
             use_advanced_pool=self._conf_get('memcache_use_advanced_pool'),
             dead_retry=self._conf_get('memcache_pool_dead_retry'),
             maxsize=self._conf_get('memcache_pool_maxsize'),
@@ -1091,7 +1097,7 @@ class AuthProtocol(_BaseAuthProtocol):
             socket_timeout=self._conf_get('memcache_pool_socket_timeout'),
         )
 
-        if security_strategy:
+        if security_strategy.lower() != 'none':
             secret_key = self._conf_get('memcache_secret_key')
             return _cache.SecureTokenCache(self.log,
                                            security_strategy,
