@@ -10,6 +10,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import abc
 import contextlib
 import hashlib
 
@@ -20,7 +21,22 @@ from keystonemiddleware.auth_token import _exceptions as exc
 from keystonemiddleware.auth_token import _memcache_crypt as memcache_crypt
 from keystonemiddleware.auth_token import _memcache_pool as memcache_pool
 from keystonemiddleware.i18n import _, _LE
-from keystonemiddleware.openstack.common import memorycache
+
+memcache = None  # module will be loaded on demand to avoid dependency
+
+
+def _create_memcache_client(*args, **kwargs):
+    """Create a new memcache client object.
+
+    This handles the lazy loaded import but also provides a point to mock out
+    in testing.
+    """
+    global memcache
+
+    if not memcache:
+        import memcache
+
+    return memcache.Client(*args, **kwargs)
 
 
 def _hash_key(key):
@@ -63,8 +79,7 @@ class _CachePool(list):
         try:
             c = self.pop()
         except IndexError:
-            # the pool is empty, so we need to create a new client
-            c = memorycache.get_client(self._memcached_servers)
+            c = _create_memcache_client(self._memcached_servers, debug=0)
 
         try:
             yield c
@@ -72,6 +87,57 @@ class _CachePool(list):
             self.append(c)
 
 
+@six.add_metaclass(abc.ABCMeta)
+class _CacheInterface(object):
+
+    def initialize(self, *args, **kwargs):
+        pass
+
+    @abc.abstractmethod
+    def store(self, key, value):
+        pass
+
+    @abc.abstractmethod
+    def store_invalid(self, key):
+        pass
+
+    @abc.abstractmethod
+    def get(self, key):
+        pass
+
+    def get_first(self, *args):
+        """Get the first cached value from many options.
+
+        :returns: token data if found else None.
+        """
+        for a in args:
+            value = self.get(a)
+
+            if value:
+                return value
+
+        return None
+
+
+class NoOpCache(_CacheInterface):
+
+    def store(self, key, value):
+        # Don't store anything
+        return None
+
+    def store_invalid(self, key):
+        # Don't store anything
+        return None
+
+    def get(self, key):
+        # Nothing to fetch from
+        return None
+
+    def get_first(self, *args):
+        # short circuit because calling get() multiple times wont help
+        return None
+
+
 class _MemcacheClientPool(object):
     """An advanced memcached client pool that is eventlet safe."""
     def __init__(self, memcache_servers, **kwargs):
@@ -84,7 +150,7 @@ class _MemcacheClientPool(object):
             yield client
 
 
-class TokenCache(object):
+class TokenCache(_CacheInterface):
     """Encapsulates the auth_token token cache functionality.
 
     auth_token caches tokens that it's seen so that when a token is re-used the
@@ -124,9 +190,14 @@ class TokenCache(object):
             return _MemcacheClientPool(self._memcached_servers,
                                        **self._memcache_pool_options)
 
-        else:
+        elif self._memcached_servers:
             return _CachePool(self._memcached_servers)
 
+        else:
+            raise RuntimeError('Trying to configure a memcache cache without '
+                               'passing any servers. This should have been '
+                               'caught.')
+
     def initialize(self, env):
         if self._initialized:
             return
@@ -266,11 +337,6 @@ class SecureTokenCache(TokenCache):
     def __init__(self, log, security_strategy, secret_key, **kwargs):
         super(SecureTokenCache, self).__init__(log, **kwargs)
 
-        security_strategy = security_strategy.upper()
-
-        if security_strategy not in ('MAC', 'ENCRYPT'):
-            msg = _('memcache_security_strategy must be ENCRYPT or MAC')
-            raise exc.ConfigurationError(msg)
         if not secret_key:
             msg = _('memcache_secret_key must be defined when a '
                     'memcache_security_strategy is defined')
